%check: 3.1.2022

clear;
clc;
addpath('Functions')
 
n=2;
Tstar=600;           %T*
tstar=Tstar-1;       %1-period shock
ibar=-1/400;         %T*-1 shock size
 
 
Y_0{1}=zeros(n,1,Tstar); %endo vars for param. 1
Y_0{2}=zeros(n,1,Tstar); %endo vars for param. 2
lam_max=zeros(4);        %record max eig--which is simply spectral radius of F in this class of models
 
 
 
 
%% Simple NK, small k
Omeg=zeros(n,n,Tstar+1); %b matrices
Ct=zeros(n,1,Tstar+1);   %a matrices 
 
%----Post-shock REE
%----STRUCTURAL PARAMETERS
        beta=.99;
        phi=1.5; 
        sigma=.2;
        kappa=.11;
 
  
 %----RE solution for t>T*: x=(y pi)'
        Ap=[1 sigma*phi;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
        Dp=zeros(2);
 
        A=Ap\Bp;
        B=Ap\Cp;
        C=Ap\Dp;
 
        [DET,FCC,OmegaK,GammaK,FK,IRs]=fmlre(A,B,C,zeros(2));
         
         
Omeg(:,:,Tstar+1)=OmegaK;   %collects b matrices
Ct(:,:,Tstar+1)=zeros(2,1); %collects a matrices
 
 
%----FG shock-horizon
        Ap=[1 0;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
         
        A=Ap\Bp;
        B=Ap\Cp;
        gam1=Ap\[-sigma*ibar;0];  %one-unit shock introduced as intercept.       
 
  
for t=1:Tstar-tstar
Omeg(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\B;
Ct(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\(gam1+A*Ct(:,:,Tstar+2-t));
end
        
%----pre-FG shock
 
        Ap=[1 0;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
         
        A=Ap\Bp;
        B=Ap\Cp;
        gam2=Ap\[0;0];        
 
  
for t=Tstar-tstar+1:Tstar
Omeg(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\B;
Ct(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\(gam2+A*Ct(:,:,Tstar+2-t));
end
   
 
for h=1:Tstar
Y_0{1}(:,:,h+1)=Ct(:,:,Tstar-h+2);
end
 
   
 
   
%% Simple NK, medium k
Omeg=zeros(n,n,Tstar+1);
Ct=zeros(n,1,Tstar+1);
 
%----Post-shock REE
%----STRUCTURAL PARAMETERS
p=.8;
 
  
%----RE solution for t>T*: x=(y pi)'
        Ap=[1 sigma*phi;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
        Dp=zeros(2);
 
        A=Ap\Bp;
        B=Ap\Cp;
        C=Ap\Dp;
 
        [DET,FCC,OmegaK,GammaK,FK,IRs]=fmlre(A,B,C,zeros(2));
         
         
Omeg(:,:,Tstar+1)=OmegaK;   %collects b matrices
Ct(:,:,Tstar+1)=zeros(2,1); %collects a matrices
 
 
%----FG shock-horizon
        Ap=[1 0;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
         
        A=Ap\Bp;
        B=Ap\Cp;
        gam1=Ap\[-sigma*ibar;0];  %one-unit shock introduced as intercept.       
 
  
Estab=max(abs(eig(p*A)));
Einstab=max(abs(eig(A)));
        
for t=1:Tstar-tstar
Omeg(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\B;
Ct(:,:,Tstar+1-t)=(eye(n)-A*Omeg(:,:,Tstar+2-t))\(gam1+A*Ct(:,:,Tstar+2-t));
end
        
%----pre-FG shock
 
        Ap=[1 0;...
            -kappa 1];
        Bp=[1 sigma;...
            0 beta];
        Cp=zeros(2);
         
        A=Ap\Bp;
        B=Ap\Cp;
        gam2=Ap\[0;0];        
 
  
for t=Tstar-tstar+1:Tstar
Omeg(:,:,Tstar+1-t)=(eye(n)-A*p*Omeg(:,:,Tstar+2-t))\B;
Ct(:,:,Tstar+1-t)=(eye(n)-A*p*Omeg(:,:,Tstar+2-t))\(gam2+p*A*Ct(:,:,Tstar+2-t));
end
   
   
   
  for h=1:Tstar
Y_0{2}(:,:,h+1)=Ct(:,:,Tstar-h+2);
end

   
   
%% Figures
phor=22;
   
   
figure(4)
% subplot(2,2,4)
%line([0 phor-2],[0 0],'Color','blue','LineStyle','--')
set(gca,'FontName', 'Times','FontSize',12);
plot(0:phor-2,squeeze(Y_0{1}(2,:,2:phor))*400,'-.k',"Linewidth",1.75)
ylabel('Inflation')
xlabel('\Delta_p')
hold on
plot(0:phor-2,squeeze(Y_0{2}(2,:,2:phor))*400,'r',"Linewidth",1.75)
hold on
ylim([0 .4])
 set(gcf,'position',[10,10,450,325])
 title('Inflation Targeting','FontSize',14,'FontName', 'Times','FontWeight','Normal')
xlabel('\Delta_p','FontSize',14,'FontName', 'Times')
ylabel('Inflation','FontSize',14,'FontName', 'Times')
legend("\eta=1", "\eta=0.8")
 
