%% Forward Guidance Analysis of the Smets and Wouters Model
% clear all;
close all;
addpath('Functions')
%% Initial Model Setup

model_name = 'usmodel_1_SW_par_FK';

eval(['dynare ', model_name, '.mod noclearall']) ;

dyn_in.options_ = options_ ;

dyn_in.M_  = M_ ;
dyn_in.oo_ = oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

SET1.M_ = dyn_in.M_ ;
SET1.oo_ = dyn_in.oo_ ;
SET1.options_ = options_ ;

out = dyn_to_str(dyn_in) ;

SET1.mats.Q = out.mats.Q ; Q = SET1.mats.Q ;
SET1.mats.G = out.mats.G ; G = SET1.mats.G ;

SET1.str_mats.A0 = out.mats.A ;
SET1.str_mats.A1 = out.mats.B ;
SET1.str_mats.B0 = out.mats.D ;
SET1.str_mats.D0 = out.mats.E ;
SET1.str_mats.D2 = out.mats.D2 ;
SET1.str_mats.Gamma = out.mats.Gamma ;

%% Extract variable details from Dynare output

n_ = M_.endo_nbr ; % Number of endogenous variables
l_ = M_.exo_nbr ;  % Number of exogenous variables
n_ = n_+1 ;        % Constant

SET.variable.n_ = n_ ; 
SET.variable.l_ = l_ ; 
SET.nparam      = M_.param_nbr ;

var_names   = cellstr(M_.endo_names) ;
param_names = cellstr(M_.param_names) ;
exo_var_names = cellstr(M_.exo_names) ;

for ii=1:n_-1
    eval(['SET.variable.',var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:l_
    eval(['SET.variable.shock.',exo_var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:M_.param_nbr
    eval(['SET.param_names.',param_names{ii},'=',int2str(ii),';']) ;
end

%% Define Theta_T* Regime
param_names = SET.param_names;
% % % Set 2: FG Under Posterior Means
M_.params(param_names.csadjcost) = 5.74;
M_.params(param_names.csigma) = 1.38;
M_.params(param_names.chabb) = 0.71;
M_.params(param_names.cprobw) = 0.7;
M_.params(param_names.csigl) = 1.83;
M_.params(param_names.cprobp) = 0.66;
M_.params(param_names.cindw) = 0.58;
M_.params(param_names.cindp) = 0.24;
M_.params(param_names.czcap) = 0.54;
M_.params(param_names.cfc) = 1.60;
%
M_.params(param_names.crpi) = 2.04;
% 
M_.params(param_names.hyper_alpha) =  1/M_.params(param_names.crpi);
%M_.params(param_names.crpi) = 2.04;
% M_.params(param_names.crr) = .81;
% M_.params(param_names.cry) = .08;
% M_.params(param_names.crdy) = .22;
% M_.params(param.names.cpie) = .62;
% M_.params(param_names.cgamma) = 0.4;
M_.params(param_names.calfa) = 0.19;
%M_.params(param_names.crhoa) = 0.5;
M_.params(param_names.crhob) = 0.95;
M_.params(param_names.crhols) = 0.22;
M_.params(param_names.crhoqs) = 0.97;
M_.params(param_names.crhoms) = 0.71;
M_.params(param_names.crhopinf) = 0.89;
M_.params(param_names.crhow) = 0.96;
M_.params(param_names.cmap) = 0.69;
M_.params(param_names.cmaw) = 0.84;


%% Recalculate model values:

[oo_.dr,info,M_,options_,oo_]  = resol(0,M_,options_,oo_) ;


dyn_in.M_  = M_ ;
dyn_in.oo_ = oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

SET1.M_ = dyn_in.M_ ;
SET1.oo_ = dyn_in.oo_ ;
SET1.options_ = options_ ;

out = dyn_to_str(dyn_in) ;

SET1.mats.Q = out.mats.Q ; Q = SET1.mats.Q ;
SET1.mats.G = out.mats.G ; G = SET1.mats.G ;

SET1.str_mats.A0 = out.mats.A ;
SET1.str_mats.A1 = out.mats.B ;
SET1.str_mats.B0 = out.mats.D ;
SET1.str_mats.D0 = out.mats.E ;
SET1.str_mats.D2 = out.mats.D2 ;
SET1.str_mats.Gamma = out.mats.Gamma ;

SET1.y_ss = (eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end)  ;
SET1.y_ss = [SET1.y_ss ; 1] ; % Constant


%% New Params and Resolve
% Set 1: FG Under the Model Priors
% M_.params(param_names.csadjcost) = 4;
% M_.params(param_names.csigma) = 1.5;
% M_.params(param_names.chabb) = .7;
% M_.params(param_names.cprobw) = 0.5;
% M_.params(param_names.csigl) = 2;
% M_.params(param_names.cprobp) = 0.5;
% M_.params(param_names.cindw) = 0.5;
% M_.params(param_names.cindp) = 0.5;
% M_.params(param_names.czcap) = 0.5;
% M_.params(param_names.cfc) = 1.25;
% M_.params(param_names.crpi) = 1.01;
M_.params(param_names.crpi) = 2.04;
% 
M_.params(param_names.hyper_alpha) = 1/M_.params(param_names.crpi);

% M_.params(param_names.crr) = 0;.75;
% M_.params(param_names.cry) =0;12;
% M_.params(param_names.crdy) = 0;.12;
% % M_.params(param.names.cpie) = .62;
% % M_.params(param_names.cgamma) = 0.4;
% M_.params(param_names.calfa) = 0.3;
% 
% % M_.params(param_names.crhoa) = 0.5;
% M_.params(param_names.crhob) = 0.5;
% M_.params(param_names.crhols) = 0.5;
% M_.params(param_names.crhoqs) = 0.5;
% M_.params(param_names.crhoms) = 0.5;
% M_.params(param_names.crhopinf) = 0.5;
% M_.params(param_names.crhow) = 0.5;
% M_.params(param_names.cmap) = 0.5;
% M_.params(param_names.cmaw) = 0.5;

[oo_.dr,info,M_,options_,oo_]  = resol(0,M_,options_,oo_) ;

dyn_in.M_  = M_ ;
dyn_in.oo_ = oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 0 ;

SET.M_ = dyn_in.M_ ;
SET.oo_ = dyn_in.oo_ ;
SET.options_ = options_ ;

out = dyn_to_str(dyn_in) ;

SET.mats.Qzlb = out.mats.Q ; Qzlb = SET.mats.Qzlb;
SET.mats.Gzlb = out.mats.G ; Gzlb = SET.mats.Gzlb;

SET.str_mats.A0 = out.mats.A;
SET.str_mats.A1 = out.mats.B;
SET.str_mats.B0 = out.mats.D;
SET.str_mats.D0 = out.mats.E;
SET.str_mats.D2 = out.mats.D2 ;
SET.str_mats.Gamma = out.mats.Gamma ;

%% Define Theta_T^a regime peg

SET.pos_of_i = SET.variable.r ; % Position of interest rate in mod file variable declaration
SET.tr_row   = 23 ;   % Row (line number) of Taylor rule in mod file
SET.zlb_val  = 0;-M_.params(SET.param_names.conster) ;   % Value of ihat at ZLB

% Structural matrices at ZLB
SET.mat_i_f_zlb = SET.str_mats ;

SET.mat_i_f_zlb.A0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A1(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.D0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.B0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A0(SET.tr_row,SET.pos_of_i) = 1 ;
SET.mat_i_f_zlb.A0(SET.tr_row,end) = -SET.zlb_val ; % Constant

SET.mat_init    = SET.str_mats ; % Structural matrices not at ZLB
SET.mat_fin     = SET.str_mats ; % Structural matrices after ZLB

% SET.zlb.Qf          = SET.mats.Q ; 
% SET.zlb.mat_init    = SET.mat_init ; 
SET.zlb.mat_i_f_zlb = SET.mat_i_f_zlb ;

A0 = SET.mat_i_f_zlb.A0;
A1 = SET.mat_i_f_zlb.A1;
B0 = SET.mat_i_f_zlb.B0;
D0 = SET.mat_i_f_zlb.D0;

%% Peg with monetary policy shock

SET.pos_of_i = SET.variable.r ; % Position of interest rate in mod file variable declaration
SET.tr_row   = 23 ;   % Row (line number) of Taylor rule in mod file
SET.zlb_val  = -1/4;-M_.params(SET.param_names.conster) ;   % Value of ihat at ZLB

% Structural matrices at ZLB
SET.mat_i_f_zlb = SET.str_mats ;

SET.mat_i_f_zlb.A0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A1(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.D0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.B0(SET.tr_row,:) = 0 ;
SET.mat_i_f_zlb.A0(SET.tr_row,SET.pos_of_i) = 1 ;
SET.mat_i_f_zlb.A0(SET.tr_row,end) = -SET.zlb_val ; % Constant

SET.mat_init    = SET.str_mats ; % Structural matrices not at ZLB
SET.mat_fin     = SET.str_mats ; % Structural matrices after ZLB

% SET.zlb.Qf          = SET.mats.Q ; 
% SET.zlb.mat_init    = SET.mat_init ; 
SET.zlb.mat_i_f_zlb = SET.mat_i_f_zlb ;

A0t = SET.mat_i_f_zlb.A0;
A1t = SET.mat_i_f_zlb.A1;
B0t = SET.mat_i_f_zlb.B0;
D0t = SET.mat_i_f_zlb.D0;

%% Simulate FG

A = A0\A1;
A0inv = inv(A0);
B = A0\B0;

exps = [10,20,30,40,50,60,70];
ll = length(exps);
for i =1:ll

T_a = 2;
T_star =T_a+1+exps(i);
T = 140;

Qat = zeros(n_,n_,T_star-T_a);
Gat = zeros(n_,l_,T_star-T_a);

Qat(:,:,end) = (A0t-B0t*SET1.mats.Q)\A1t;
Gat(:,:,end) = SET1.mats.G;

% Sequence of reduced form matrices
for t = T_star-T_a-1:-1:1
    Qat(:,:,t) = (eye(n_)-B*Qat(:,:,t+1))\A;
    Gat(:,:,t) = (eye(n_)-B*Qat(:,:,t+1))\A0inv*D0;
end

Qt = zeros(n_,n_,T);
Gt = zeros(n_,l_,T);


for t = 1:T_a-1
    Qt(:,:,t) = SET1.mats.Q;
    Gt(:,:,t) = SET1.mats.G;
end
for t = T_a:T_star-1
    Qt(:,:,t) = Qat(:,:,t-T_a+1);
    Gt(:,:,t) = Gat(:,:,t-T_a+1);
end

   Qt(:,:,T_star) = Qat(:,:,end);
   Gt(:,:,T_star) = (A0 - B0*SET1.mats.Q)\D0;

for t = T_star:T
    Qt(:,:,t) = SET1.mats.Q;
    Gt(:,:,t) = SET1.mats.G;
end


y_non = zeros(n_,T);
y_non(:,1) = SET1.y_ss;

for t = 2:T
    y_non(:,t) = Qt(:,:,t) * y_non(:,t-1);
end

figure(1)
subplot(3,1,1)
plot(y_non(SET.variable.pinfobs,:)*4,'Linewidth',2)
hold on
ylabel('Inflation','FontName', 'Times New Roman','FontSize',16)
% xlabel('\Delta_p','FontName', 'Times New Roman')
subplot(3,1,2)
plot(y_non(SET.variable.y,:),'Linewidth',2)
hold on
ylabel('Output','FontName', 'Times New Roman','FontSize',16)
% xlabel('\Delta_p','FontName', 'Times New Roman')
subplot(3,1,3)
plot(y_non(SET.variable.robs,:)*4,'Linewidth',2)
hold on
ylabel('Interest Rate','FontName', 'Times New Roman','FontSize',16)
% xlabel('\Delta_p','FontName', 'Times New Roman')
axis([0 T 3.5 6])

end

